/******************************************************************************
 * @file     main.c
 * @version  V1.00
 * $Revision: 5 $
 * $Date: 13/10/30 8:20a $ 
 * @brief    GPIO project for Mini51 series MCU
 *
 * @note
 * Copyright (C) 2013 Nuvoton Technology Corp. All rights reserved.
*****************************************************************************/   
#include <stdio.h>
#include "mini51series.h"
#include "gpio.h"
#include "i2c_software_gpio.h"
#include "i2c_hardware.h"
uint32_t count = 0;
uint8_t I2C_Tx_Data[6];
uint8_t I2C_Rx_Data[6];

void SYS_Init(void)
{
    /* Unlock protected registers */
    SYS_UnlockReg();
    
/*---------------------------------------------------------------------------------------------------------*/
/* Init System Clock                                                                                       */
/*---------------------------------------------------------------------------------------------------------*/

   
		/*Disable XTL CLK (internal clock)*/
		CLK->PWRCON &=~CLK_PWRCON_XTLCLK_EN_Msk;
    /* Enable internal 22.1184MHz/10KHz CLK */
    CLK->PWRCON |=  (CLK_PWRCON_IRC22M_EN_Msk|CLK_PWRCON_IRC10K_EN_Msk);		
    /* Waiting for clock ready */
    CLK_WaitClockReady( CLK_CLKSTATUS_IRC22M_STB_Msk|CLK_CLKSTATUS_IRC10K_STB_Msk);

    /* Switch HCLK clock source to IRC22M */
    CLK_SetHCLK(CLK_CLKSEL0_HCLK_S_IRC22M,CLK_CLKDIV_HCLK(1));
    
    /* STCLK to IRC22M */
    CLK_SetSysTickClockSrc(CLK_CLKSEL0_STCLK_S_IRC22M_DIV2);
		
    /* Enable IP clock */       
		/* Enable ADC and PWM45 clock */	
    CLK_EnableModuleClock(UART_MODULE); //PIN2
		CLK_EnableModuleClock(I2C_MODULE);	//PIN7 8
		CLK_EnableModuleClock(TMR0_MODULE);
		
    //CLK->APBCLK |= CLK_APBCLK_ADC_EN_Msk;
    /* ADC clock source is 22.1184MHz, set divider to (3 + 1), ADC clock is 22.1184/4 MHz */		
		//CLK_SetModuleClock(ADC_MODULE,CLK_CLKSEL1_ADC_S_IRC22M,CLK_CLKDIV_ADC(4));
    /* Select IP clock source */
    //CLK_SetModuleClock(UART_MODULE,CLK_CLKSEL1_UART_S_IRC22M,CLK_CLKDIV_UART(1));		
		/* Select TIM0 clock source from internal 10K */
    //CLK_SetModuleClock(TMR0_MODULE,CLK_CLKSEL1_TMR0_S_IRC10K,CLK_CLKDIV_HCLK(1));
		CLK->CLKSEL1=0xA200010C;   //set UART/ADC/PWM use internal clock
		CLK->CLKDIV=0x30000;       //ADC @22.114/4 MHz
		
/*---------------------------------------------------------------------------------------------------------*/
/* Init I/O Multi-function                                                                                 */
/*---------------------------------------------------------------------------------------------------------*/
    /* Set P1 multi-function pins for UART RXD and TXD */
    SYS->P1_MFP &= ~(SYS_MFP_P12_Msk | SYS_MFP_P13_Msk);
    SYS->P1_MFP |= (SYS_MFP_P12_RXD | SYS_MFP_P13_TXD);  

		/*Set P3.4 P3.5 as I2C*/
		SYS->P3_MFP |= SYS_MFP_P34_SDA;
		SYS->P3_MFP |= SYS_MFP_P35_SCL;

		/* Analog pin OFFD(OFF Digital Enable) to prevent leakage */
    P1->OFFD |= (1 << 2) << GPIO_OFFD_OFFD_Pos;    
		P1->OFFD |= (1 << 4) << GPIO_OFFD_OFFD_Pos;    
		P1->OFFD |= (1 << 5) << GPIO_OFFD_OFFD_Pos;  

    /* To update the variable SystemCoreClock */
    SystemCoreClockUpdate();
            
    /* Lock protected registers */
    SYS_LockReg();    
}

 
void UART_Init(void)
{
/*---------------------------------------------------------------------------------------------------------*/
/* Init UART                                                                                               */
/*---------------------------------------------------------------------------------------------------------*/
    /* Reset IP */
    SYS_ResetModule(SYS_IPRSTC2_UART_RST_Msk);
    
    /* Configure UART and set UART Baudrate */
    UART_Open(UART, 9600);
    
}

void delay_loop(void)
{
    __IO uint32_t j;
    P47 ^= 1;
    for(j=0;j<60000;j++);
    for(j=0;j<60000;j++);
    for(j=0;j<60000;j++);
    for(j=0;j<60000;j++);
}

void I2CInit(void)
{
	I2C_Tx_Data[0] = 0x75;
	printf("InitI2C_HW \r");
	InitI2C_HW();
	printf("I2C_SW_Open \r");
	I2C_SW_Open(100000);
	printf("I2C_Send Data\r");
	I2C_SW_Send(0xD0,I2C_Tx_Data,1);
  CLK_SysTickDelay(5000);
  I2C_SW_Get(0xD0,I2C_Rx_Data,1);
	printf("%x%x%x%x%x%x \r",I2C_Rx_Data[0],I2C_Rx_Data[1],I2C_Rx_Data[2],I2C_Rx_Data[3],I2C_Rx_Data[4],I2C_Rx_Data[5]);
}

int main(void)
{
    /* If define INIT_SYSCLK_AT_BOOTING in system_M051Series.h, HCLK will be set to 50MHz in SystemInit(void)*/
    if(SYS->RegLockAddr == 1) // In end of main function, program issued CPU reset and write-protection will be disabled.
        SYS_LockReg();
    
    /* Init System, IP clock and multi-function I/O */
    SYS_Init(); //In the end of SYS_Init() will issue SYS_LockReg() to lock protected register. If user want to write protected register, please issue SYS_UnlockReg() to unlock protected register.    

    /* Init UART for printf */
    UART_Init();

    printf("+---------------------------------+\n");
    printf("|    Mini51 NuTiny Sample Code    |\n");
    printf("+---------------------------------+\n");

		I2CInit();
		while(1)
		{
			
		}
		
		
   
    while(1)
    {
			count++;
      printf("Count = %3d \n",count);
      delay_loop();
		}

}

#ifdef USE_ASSERT
/**
  * @brief  The function prints the source file name and line number where the assert_param() error
      occurs, and then stops in an infinite loop. User can add his own codes here if necessary.
  * @param  file: source file name.
  * @param  line: line number.
  * @retval None.
  */

void assert_error(uint8_t * file, uint32_t line)
{
    /*en: Infinite loop */
    while(1) ;

}
#endif
/*** (C) COPYRIGHT 2013 Nuvoton Technology Corp. ***/
